//  Beispiel 6.6.6 "LEDs via Website schalten"
//   
//  Die Bilddateien sowie die CSS und JavaScript Dateien aus dem Verzeichnis "data"
//  (siehe Unterverzeichnis im Beispiel_6.6.6), müssen mit dem "ESP8266 Sketch Data Upload" 
//  Tool übertragen werden. Dieses erscheint nach der Installation im "Werkzeuge"-Menü.
//  
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>

#include "FS.h"

int counter = 0;

int gpio13 = 13;
int gpio14 = 14;
String gpio13Name = "GPIO13";
String gpio14Name = "GPIO14";
int dOn = HIGH;
int dOff = LOW;

void setup()
{
  IoT_Init(true);
  IoT_WLANconnect("(Hotspot Name)", "(Hotspot Password)");          // An dieser Stelle die eigenen Zugangsdaten eintragen
  IoT_NTPinit();

  IoT_WebServer.on("/", handleRoot); // Wenn der Browser direkt auf das Stammverzeichnis zugreift, führe die Funktion handleRoot (siehe unten) aus.
  /*  Folgende Einträge bestimmen, welche Funktionen ausgeführt werden wenn man bestimmte Links im Browser aufruft,
   *  falls z.B. der IoT Brick die IP Adresse 192.168.1.153 vom DHCP Server erhalten hat, dann würde das Ausrufen der 
   *  Adresse http://192.168.1.153/img/brklogo.png dazu führen, dass die Funktion handleImgLogo ausgeführt wird, die wiederum 
   *  die Bilddatei brklogo.png aus dem internen Dateisystem (SPIFFS) liest und an den Webserver liefert.  
   */
   
  // JS (Javascript)
  IoT_WebServer.on("/js/jquery", handleJsJquery);
  IoT_WebServer.on("/js/bootstrap", handleJsBootstrap);
  IoT_WebServer.on("/js/bootstrap-switch", handleJsBootstrapSwitch);
  // CSS (Cascaded Style Sheets 
  IoT_WebServer.on("/css/bootstrap", handleCssBootstrap);
  IoT_WebServer.on("/css/bootstrap-switch", handleCssBootstrapSwitch);
  // Images (Bilder)
  IoT_WebServer.on("/img/bg.png", handleImgBg);
  IoT_WebServer.on("/img/brklogo.png", handleImgLogo);

  // WebServer Handles für die GPIO Schaltfunktionen
  IoT_WebServer.on("/switch13-on", handleGpio13On);
  IoT_WebServer.on("/switch13-off", handleGpio13Off);
  IoT_WebServer.on("/switch14-on", handleGpio14On);
  IoT_WebServer.on("/switch14-off", handleGpio14Off);
  
  IoT_WebServer.begin();
  Serial.println("HTTP server started");
  setupPins();
}

void loop()
{
  IoT_Idle();

  IoT_WebServer.handleClient(); //Bediene die HTTP-Anfragen

  IoT_Idle();

  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayWLANstatus();
    if (counter % 10 == 0) //Infos seriell nicht zu oft ausgeben (ca. jede Sekunden, nur wenn der Button gedrückt wird)
    {
      Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
      Serial.print("WiFi is ");
      Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
      Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    }
  }
  else
  {
    IoT_Idle();
    if (counter % 25 == 0)                                          // NTP Event alle 2 Sekunden auslesen
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent-Infos im Terminal ausgeben
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_NTPvalid())                                             // Prüfen, ob eine gültige Uhrzeit aus dem Internet empfangen wurde
    {
      IoT_DisplayDrawText(63, 0, IoT_NTPtime());
      IoT_DisplayDrawText(63, 15, IoT_NTPdate());
      IoT_Idle();
    }
    else                                                            // "Bitte warten..." zentriert anzeigen
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                      // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Bitte");
      IoT_DisplayDrawText(64, 34, "warten...");
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  delay(100);                                                       // 100 ms. warten
  counter++;
}

void handleRoot()
{
  String content;
  content = "<!DOCTYPE html>";
  content += "<html>";
  content += "<head>";
  content += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">";  // wichtig, damit das Grad-Zeichen "°" korrekt dargestellt wird
  content += "<meta http-equiv=\"cache-control\" content=\"max-age=0\">";
  content += "<meta http-equiv=\"cache-control\" content=\"no-cache\">";
  content += "<meta http-equiv=\"expires\" content=\"0\">";
  content += "<meta http-equiv=\"expires\" content=\"Tue, 01 Jan 1980 1:00:00 GMT\">";
  content += "<meta http-equiv=\"pragma\" content=\"no-cache\">";
  content += "<meta http-equiv=\"Content-Language\" content=\"de_DE\">";
  content += "<meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">";
  content += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1, user-scalable=no\">";
  content += "<meta name=\"description\" content=\"BrickRknowledge IoT Brick-Website\">";
  content += "<meta name=\"author\" content=\"ALLNET\">";
  content += "<title>IoT Brick via WLAN " + IoT_WLANssid() + " schalten</title>";
  content += "<link rel=\"stylesheet\" href=\"/css/bootstrap\">";
  content += "<link rel=\"stylesheet\" href=\"/css/bootstrap-switch\">";
  content += "<style>body{background-image:url(/img/bg.png);margin:0;padding:20px;background-size:100% auto;background-repeat:no-repeat;font-family:\"Helvetica Neue\",Helvetica,Arial,sans-serif;font-size:14px;line-height:1.5;color:#333;background-color:#fff}.col-sm-2{margin-top:20px}.img-thumbnail{border:0}</style>";
  content += "</head>";
  content += "<body>";
  content += "<div class=\"container-fluid\">";
  content += "<div class=\"row\">";
  content += "<div class=\"col-sm-2\"><img class=\"img-thumbnail\" src=\"/img/brklogo.png\"></div>";
  content += "</div>";
  content += "<div class=\"row\">";
  content += "<div class=\"col-sm-2\"><input type=\"checkbox\" id=\"switch14\" data-label-text=\""+gpio14Name+"\" data-label-width=\"120\"></div>";
  content += "<div class=\"col-sm-2\"><input type=\"checkbox\" id=\"switch13\" data-label-text=\""+gpio13Name+"\" data-label-width=\"120\"></div>";
  content += "</div>";
  content += "</div>";
  content += "<script src=\"/js/jquery\"></script>";
  content += "<script src=\"/js/bootstrap\"></script>";
  content += "<script src=\"/js/bootstrap-switch\"></script>";
  content += "<script type=\"text/javascript\">";
  content += "$('input[type=\"checkbox\"]').bootstrapSwitch({onSwitchChange:function(){$.ajax({url:'/'+$(this).prop('id')+'-'+($(this).prop('checked')?'on':'off')});}});";
  content += "var setAdc=function(){$.ajax({url:'/adc'}).done(function(data){data=data||{};if(data.hasOwnProperty('data')){$('#adc').text(data.data);}}).fail(function(jqxhr,textStatus,error){}).always(function(){setTimeout(setAdc,1000);});};setAdc();";
  content += "</script>";
  content += "</body>";
  content += "</html>";
  IoT_WebServer.send(200, "text/html", content);   // HTTP-Statuscode 200 = OK (Anfrage wurde erfolgreich bearbeitet)
  IoT_Idle();
}

/*****************************************************************************************
 * Für alle die sich schon gut mit HTML auskennen und selbst die Website modifizieren 
 * möchten, finden im Folgenden den HTML-Code aus der Funktion handleRoot() "im Klartext". 
 * Am besten du kopiert ihn in einen HTML-Editor deiner Wahl.
 * ***************************************************************************************
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<meta http-equiv="cache-control" content="max-age=0">
<meta http-equiv="cache-control" content="no-cache">
<meta http-equiv="expires" content="0">
<meta http-equiv="expires" content="Tue, 01 Jan 1980 1:00:00 GMT">
<meta http-equiv="pragma" content="no-cache">
<meta http-equiv="Content-Language" content="de_DE">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1, user-scalable=no"><meta name="description" content="BrickRknowledge IoT Brick Website">
<meta name="author" content="ALLNET">
<title>IoT Brick via "+network+" schalten</title>
<link rel="stylesheet" href="/css/bootstrap">
<link rel="stylesheet" href="/css/bootstrap-switch">
<style>
body{background-image:url(/img/bg.png);margin:0;padding:20px;background-size:100% auto;background-repeat:no-repeat;font-family:"Helvetica Neue",Helvetica,Arial,sans-serif;font-size:14px;line-height:1.5;color:#333;background-color:#fff}.col-sm-2{margin-top:20px}.img-thumbnail{border:0}
</style>
</head>
<body>
<div class="container-fluid">
<div class="row">
<div class="col-sm-2">
<img class="img-thumbnail" src="/img/brklogo.png">
</div>
</div>
<div class="row">
<div class="col-sm-2">
<input type="checkbox" id="switch13" data-label-text="gpio13Name" data-label-width="120">
</div>
<div class="col-sm-2">
<input type="checkbox" id="switch14" data-label-text="gpio14Name+" data-label-width="120">
</div>
</div>
</div>
<script src="/js/jquery"></script>
<script src="/js/bootstrap"></script>
<script src="/js/bootstrap-switch"></script>
<script type="text/javascript">$('input[type="checkbox"]').bootstrapSwitch({onSwitchChange:function(){$.ajax({url:'/'+$(this).prop('id')+'-'+($(this).prop('checked')?'on':'off')});}});var setAdc=function(){$.ajax({url:'/adc'}).done(function(data){data=data||{};if(data.hasOwnProperty('data')){$('#adc').text(data.data);}}).fail(function(jqxhr,textStatus,error){}).always(function(){setTimeout(setAdc,1000);});};setAdc();</script>
</body>
</html>
******************************
*    Ende des HTML-Codes     *
******************************
*/


// GPIO 13 und 14 werden als Ausgang konfiguriert um die LEDs schalten zu können
 void setupPins()
{
  pinMode(gpio13, OUTPUT);
  pinMode(gpio14, OUTPUT);
  digitalWrite(gpio13, dOff);
  digitalWrite(gpio14, dOff);
}

//Handle-Funktion für GPIO13 ON
void handleGpio13On()
{
  digitalWrite(gpio13, dOn);
  //IoT_WebServer.send(200, "text/html", gpio13Name + " on");
  Serial.println("GPIO13 -> ON");
}

//Handle-Funktion für GPIO13 OFF
void handleGpio13Off()
{
  digitalWrite(gpio13, dOff);
  //IoT_WebServer.send(200, "text/html", gpio13Name + " off");
  Serial.println("GPIO13 -> OFF");
}

//Handle-Funktion für GPIO14 ON
void handleGpio14On()
{
  digitalWrite(gpio14, dOn);
  //IoT_WebServer.send(200, "text/html", gpio14Name + " on");
  Serial.println("GPIO14 -> ON");
}

//Handle-Funktion für GPIO14 OFF
void handleGpio14Off()
{
  digitalWrite(gpio14, dOff);
  //IoT_WebServer.send(200, "text/html", gpio14Name + " off");
  Serial.println("GPIO14 -> OFF");
}

/*
 * Folgende Funktionen sind notwendig um diverse Dateien die JQuery und JavaScript Funktionen
 * beinhalten aus dem internen Filesystem (SPIFFS definiert in FS.h) des IoT Bricks auszulesen 
 * und dem WebServer zur Verfügung zu stellen.
 */

void handleJsJquery() {
  String path = "/jquery.js.gz";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f)
  {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "application/javascript");
    f.close();
  }
  else
  {
    Serial.println("JsJquery open failed");
  }
  SPIFFS.end();
}

void handleJsBootstrap()
{
  String path = "/bootstrap.js.gz";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "application/javascript");
    f.close();
  } else {
    Serial.println("JsBootstrap open failed");
  }
  SPIFFS.end();
}

void handleJsBootstrapSwitch()
{
  String path = "/bootstrap-switch.js.gz";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "application/javascript");
    f.close();
  } else {
    Serial.println("JsBootstrapSwitch open failed");
  }
  SPIFFS.end();
}

void handleCssBootstrap()
{
  String path = "/bootstrap.css.gz";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "text/css");
    f.close();
  } else {
    Serial.println("CssBootstrap open failed");
  }
  SPIFFS.end();
}

void handleCssBootstrapSwitch()
{
  String path = "/bootstrap-switch.css.gz";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "text/css");
    f.close();
  } else {
    Serial.println("CssBootstrapSwitch open failed");
  }
  SPIFFS.end();
}

void handleImgBg()
{
  String path = "/bg.png";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "image/png");
    f.close();
  } else {
    Serial.println("ImgBg open failed");
  }
  SPIFFS.end();
}

void handleImgLogo()
{
  String path = "/brklogo.png";
  SPIFFS.begin();
  File f = SPIFFS.open(path, "r");
  if(f) {
    f.setTimeout(0);
    size_t sent = IoT_WebServer.streamFile(f, "image/png");
    f.close();
  } else {
    Serial.println("ImgBrklogo open failed");
  }
  SPIFFS.end();
}


